import type { ColorValue, StyleProp, TextStyle } from 'react-native'
import type { ThemeInput, ThemeRegistration } from 'shiki'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { getRNStylesFromShikiStyle } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface TokenStyle extends Omit<TextStyle, 'color' | 'fontWeight'> {
  color?: ColorValue
  fontStyle?: 'normal' | 'italic'
  fontWeight?: FontWeight
  fontFamily?: string
}

interface Styles {
  [key: string]: TokenStyle
}

type FontWeight = 'normal' | 'bold' | '100' | '200' | '300' | '400' | '500' | '600' | '700' | '800' | '900'

// Helper function to extract theme registration from Shiki theme input
function getThemeRegistration(theme: ThemeInput): ThemeRegistration {
  if ('default' in theme) {
    return theme.default
  }
  return theme as ThemeRegistration
}

const baseStyles = StyleSheet.create({
  scrollView: {
    flex: 1,
    minHeight: 20,
  },
  lineContainer: {
    flexDirection: 'row',
    flexWrap: 'wrap',
    paddingHorizontal: 16,
    paddingVertical: 8,
  },
  token: {
    fontFamily: monospaceFont,
    fontSize: 14,
  },
  container: {
    flex: 1,
  },
})

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14 }) => {
  const stylesheet = useMemo(() => {
    const themeRegistration = getThemeRegistration(theme)
    const styles = getRNStylesFromShikiStyle(themeRegistration)
    return StyleSheet.create<Styles>({
      ...styles,
      editor: {
        backgroundColor: themeRegistration.colors?.['editor.background'] || '#1e1e1e',
      },
      token: {
        ...baseStyles.token,
        fontSize,
        color: themeRegistration.colors?.['editor.foreground'] || '#d4d4d4',
      },
    })
  }, [theme, fontSize])

  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  const renderToken = (token: any, index: number, lineIndex: number) => {
    const styles: StyleProp<TokenStyle>[] = [stylesheet.token]

    // Add token-specific styles from stylesheet based on scope
    const tokenScopes = Array.isArray(token.scope) ? token.scope : [token.scope]
    tokenScopes.forEach((scope: string) => {
      if (stylesheet[scope]) {
        styles.push(stylesheet[scope])
      }
    })

    // Add token-specific styles from the token itself
    const tokenStyles: TokenStyle = {}
    if (token.color) {
      tokenStyles.color = token.color
    }
    if (token.fontStyle === 'italic') {
      tokenStyles.fontStyle = 'italic'
    }
    if (token.fontWeight === 'bold') {
      tokenStyles.fontWeight = 'bold'
    }

    if (Object.keys(tokenStyles).length > 0) {
      styles.push(tokenStyles)
    }

    const finalStyle = StyleSheet.flatten(styles)

    return (
      <Text key={`${lineIndex}-${index}`} style={finalStyle}>
        {token.content.replace(/ /g, '\u00A0')}
      </Text>
    )
  }

  const renderLine = (line: any[], lineIndex: number) => (
    <View key={lineIndex} style={baseStyles.lineContainer}>
      {line.map((token, tokenIndex) => renderToken(token, tokenIndex, lineIndex))}
    </View>
  )

  return (
    <ScrollView horizontal showsHorizontalScrollIndicator={Platform.OS !== 'web'} style={[baseStyles.scrollView, stylesheet.editor]} contentContainerStyle={baseStyles.container}>
      {tokens.map((line, index) => renderLine(line, index))}
    </ScrollView>
  )
}

export default SyntaxHighlighter
